#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
# Construct a Science Data Model from a set of SDM XML tables

# Consider doing validation on the presence of the BDF files
# General validation of the SDM

import logging
import os
import sys

from lxml import objectify as obj

# Verify the contents of the SDM and cross-verify with BDFs
from lxml.etree import XMLSyntaxError

logger = logging.getLogger("ingestion")


class SDM:
    def __init__(self, fileset, sdmbasepath=".", bdfbasepath=None):
        self.fileset = fileset
        self.sdmtables = buildsdmtables(self, os.path.join(sdmbasepath, fileset))
        self.bdffiles = addbdffiles(self)


class SDMTable:
    def __init__(self, tablename, entityid, size):
        self.tablename = tablename
        self.entityid = entityid
        self.size = size


class SDMXMLTable(SDMTable):
    def __init__(self, tablename, entityid, table, size):
        SDMTable.__init__(self, tablename, entityid, size)
        self.table = table


class SDMBinaryTable(SDMTable):
    def __init__(self, tablename, entityid, size):
        SDMTable.__init__(self, tablename, entityid, size)


class BDFFile:
    def __init__(self, entityid, filesize):
        self.entityid = entityid
        self.filesize = filesize


def buildsdm(fileset="abc23", sdmbasepath=".", bdfbasepath="."):
    sdm = SDM(fileset, sdmbasepath, bdfbasepath)
    return sdm


def buildsdmtables(sdm, sdmbasepath):
    sdmtables = {}

    # Get the SDM Table files
    sdmtree = obj.parse(os.path.join(sdmbasepath, "ASDM.xml"))
    sdmroot = sdmtree.getroot()
    if sdmroot.find("Table") is None:
        logger.error("ASDM.xml has no table entries in {}".format(sdmbasepath))
        sys.exit(-1)
    for table in sdmroot.Table:
        tablepath = os.path.join(sdmbasepath, str(table.Name))
        if os.path.isfile(tablepath + ".xml"):
            # Don't parse if it's the old format SysPower.xml due to the size
            if table.Name == "SysPower":
                sdmtabletree = None
            else:
                try:
                    sdmtabletree = obj.parse(tablepath + ".xml")
                    sdmxmltable = SDMXMLTable(
                        table.Name.text, table.Entity.get("entityId"), sdmtabletree, os.path.getsize(tablepath + ".xml")
                    )
                    sdmtables[table.Name] = sdmxmltable
                except XMLSyntaxError:
                    pass
        elif os.path.isfile(tablepath + ".bin"):
            sdmbintable = SDMBinaryTable(
                table.Name.text, table.Entity.get("entityId"), os.path.getsize(tablepath + ".bin")
            )
            sdmtables[table.Name] = sdmbintable
        else:
            logger.error("SDM Table {} does not exist for {}".format(table.Name, sdm.fileset))
    asdmtable = SDMXMLTable("ASDM", sdmroot.Entity.get("entityId"), sdmtree, os.path.getsize(sdmbasepath + "/ASDM.xml"))
    sdmtables["ASDM"] = asdmtable

    return sdmtables


def addbdffiles(sdm):
    bdffiles = []
    maintableroot = sdm.sdmtables["Main"].table.getroot()
    if len(maintableroot.findall("row")) == 0:
        logger.error("No BDF files to be loaded for {}".format(sdm.fileset))
        return bdffiles
    for row in maintableroot.row:
        if row.find("dataUID") is not None:
            entityid = row.dataUID.EntityRef.get("entityId")
        elif row.find("dataOid") is not None:
            entityid = row.dataOid.EntityRef.get("entityId")
        else:
            logger.critical("Invalid XML element in Main")
        if row.find("dataSize") is not None:
            filesize = row.dataSize
        else:
            # TODO for legacy data, find a better way
            filesize = 0
        bdffile = BDFFile(entityid, filesize)
        bdffiles.append(bdffile)
    return bdffiles


if __name__ == "__main__":
    sdm = buildsdm("15A-017.sb30393593.eb30429744.57067.21779167824", "/Users/rlively/Development/python/testdata", ".")
