#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import glob
import logging
import os
import tarfile

LOG = logging.getLogger("thumbnail_cheater")


class Weblog:
    """A class to encapsulate the extraction of data from the initial page
    of a CASA Pipeline weblog.  Uses BeautifulSoup for the processing.

    Given the path to a file & it's name: extract & scrape the basic information
    from the index.html file.

    Initially: required the construction of a full path to the index file file as a string:
    mylog = new Weblog("/export/home/earth/image_scratch/pipeline-20180222T184022/html/index.html")

    Then use the methods to extract the relevant information from the file.
    """

    # -------------------------------------------------------------------------
    # Internal Use Methods
    # -------------------------------------------------------------------------

    def __init__(self, path_to_file, filename):
        """Open the weblog's index.html file and perform parse the document."""
        self.indexFilePath = self.find_index_file(path_to_file, filename)
        # self.thumbnailImages = self.collect_potential_quicklook_thumbnails(self.indexFilePath)

    def find_index_file(self, products_dir, weblog_file_name):
        weblog_tar_file = tarfile.open(os.path.join(products_dir, weblog_file_name))
        weblog_tar_file.extractall(path=products_dir)
        weblog_index_html = glob.glob(products_dir + "/pipeline-*/html/index.html")
        return weblog_index_html[0]

    # -------------------------------------------------------------------
    # Mixed, probably mostly internal.
    # -------------------------------------------------------------------

    def collect_potential_quicklook_thumbnails(self, imageFilePath):
        """
        Retrieve the list of thumnbnail images from the appropriate stage for quicklook imaging:
        :param imageFilePath: path to the weblog
        :return: list of thumbnail images
        """
        return glob.glob(os.path.dirname(imageFilePath) + "/stage7/thumbs/*.sky.png", recursive=True)

    def collect_potential_continuum_thumbnails(self, imageFilePath):
        """
        Retrieve the list of thumnbnail images in the weblog stage for continuum imaging:
        :param imageFilePath: path to the weblog
        :return: list of thumbnail images
        """
        return glob.glob(os.path.dirname(imageFilePath) + "/stage10/thumbs/*.sky.png", recursive=True)

    def collect_potential_cube_thumbnails(self, imageFilePath):
        """
        Retrieve the list of thumnbnail images in the weblog's cube imaging stage
        :param imageFilePath: path to the weblog
        :return: list of thumbnail images
        """
        return glob.glob(os.path.dirname(imageFilePath) + "/stage12/*.sky.png", recursive=True)

    # --------------------------------------------------------------------
    #  External Methods
    # --------------------------------------------------------------------

    def find_quicklook_thumbnail_for(self, filename):
        """
        Get the thumbnail image from the weblog for the specified filename
        :param filename: FITS image to search for thumbnail
        :return: thumbnail image path in the weblog
        """

        thumbnailImages = self.collect_potential_quicklook_thumbnails(self.indexFilePath)

        for thumbnailImage in thumbnailImages:
            # logger.info('Checking if {} is a thumbnail for {}'.format(os.path.basename(thumbnailImage), filename.replace('-', '_').replace('+', '_').replace('fits', 'sky.png')))
            if os.path.basename(thumbnailImage) == filename.replace("-", "_").replace("+", "_").replace(
                "fits", "sky.png"
            ):
                return thumbnailImage

        LOG.error("No thumbnail found for image: {}".format(filename))
        return None

    def find_continuum_thumbnail_for(self, filename):
        """
        Get the thumbnail image from the weblog for the specified filename
        :param filename: FITS image to search for thumbnail
        :return: thumbnail image path in the weblog
        """

        thumbnailImages = self.collect_potential_continuum_thumbnails(self.indexFilePath)

        # Three generic replacements:
        expected_thumb = filename.replace("-", "_").replace("+", "_").replace("fits", "sky.png")

        # And one based upon the filename (because there are two variants in what comes out of the
        # continuum imaging step:
        if "tt" in expected_thumb:
            expected_thumb = expected_thumb.replace("I.tt", "I.iter1.image.tt")
        else:
            expected_thumb = expected_thumb.replace("I.pbcor", "I.iter1.image.pbcor")

        LOG.debug(f"Searching for a file named: {expected_thumb}")

        for thumbnailImage in thumbnailImages:

            currentThumb = os.path.basename(thumbnailImage)
            # there's an extra portion stuffed in the beginning of the name, it's best handled here:
            comparisonThumb = currentThumb.replace(".s10_0", "")
            LOG.debug(f"\tcomparing to: {currentThumb}({comparisonThumb})")

            if comparisonThumb == expected_thumb:
                return thumbnailImage

        LOG.error("No thumbnail found for image: {}".format(filename))
        return None

    def find_cube_thumbnail_for(self, filename):
        """
        Get the thumbnail image from the weblog for the specified filename.  There are several
        transformations we need to make in order to match the relevant file.

        :param filename: FITS image to search for thumbnail
        :return: full path to the (unaltered) thumbnail image for copying.
        """

        thumbnailImages = self.collect_potential_cube_thumbnails(self.indexFilePath)
        # The thumbnail images in the weblog don't exactly match the resultant files.
        # So, let's meet them halfway:
        expected_thumb = (
            filename.replace("-", "_")
            .replace("+", "_")
            .replace("fits", "sky.png")
            .replace("I.pbcor", "I.iter1.image.pbcor")
        )
        LOG.debug(f"Searching for a file named: {expected_thumb}")

        for thumbnailImage in thumbnailImages:
            currentThumb = os.path.basename(thumbnailImage)
            # there's an extra portion stuffed in the beginning of the name, it's best handled here:
            comparisonThumb = currentThumb.replace(".s12_0", "")
            LOG.debug(f"\tcomparing to: {currentThumb}({comparisonThumb})")

            if comparisonThumb == expected_thumb:
                return thumbnailImage

        LOG.error("No thumbnail found for image: {}".format(filename))
        return None
