#!/usr/bin/env bash
#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

#
#  A script from the archive repo adapted for Workspaces
#
#  Old archive description:
#       A replacement for the old qaarchive script used by the
#       data analysts.  This script will tar & zip the pipeline
#       weblog into WEBLOG.tar.gz and then collect everything but
#       FITS files in the products directory from a CIPL run into
#       a single tar file (naming convention TBD) that is created
#       in a storage directory for ingestion.
#
#  Arguments:
#       1: Working directory to be worked upon
#       2: Project code of observation
#       3: Start time of running this script
#       4: CAPO profile
#

# Basics: Path modification
set -o errexit -o nounset -o xtrace

#
# Command-line arguments
#

# Dir that identifies particular workflow
WORKFLOW_DIR=$1;shift
# Metadata for the output tar's naming scheme
SDM_ID=$1;shift
CAL_PROCESSING_DATETIME=$1;shift

# Get the spool, staging and storage paths from CAPO
SPOOL_DIR=$(./pycapo -P "${CAPO_PROFILE}" -q edu.nrao.workspaces.ProcessingSettings.rootDirectory)
STAGING_DIR=$(./pycapo -P "${CAPO_PROFILE}" -q edu.nrao.workspaces.IngestionSettings.stagingDirectory)
STORAGE_DIR=$(./pycapo -P "${CAPO_PROFILE}" -q edu.nrao.workspaces.IngestionSettings.storageDirectory)

# File name for output tar of calibration products
#
# Naming scheme: <sdmId>.<calProcessingDatetime>.caltables.tar
#         a.k.a. <project_code>.<sb>.<eb>.<obs_time_in_mjd>.<cal_processing_timestamp>.caltables.tar
FILENAME="${SDM_ID}"."${CAL_PROCESSING_DATETIME}".caltables

#
# For the ability to reproduce results, we'll want the PPR.xml file.  Ensure it is
# included in the products/ directory:
#
# Check for spool/<workflow_dir>/products/*.pprequest.xml. If it exists, do nothing. If not, move base_dir/PPR.xml to products
if test -n "$(find "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products -maxdepth 1 -name '*.pprequest.xml' -print -quit)"
then
    cp "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/PPR.xml "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products/PPR.xml
fi

#
# The VLASS project wants the flux.csv file.  Check if it's here, if not, check for it in
# the working directory parallel to this one.  Don't fail if we can't find it, however (so
# we minimize the disruption to the CIPL system).
#
# HERE USED TO LIE VLASS-SPECIFIC CODE

#
# Both the pipeline-YYYMMDDTHHMMSS directory and weblog.tgz should exist.  We prefer the
# directory (in case of updates/edits), but fall back on the tgz file.
#
# Check that they're both home, as we expect
WEBLOG_DIR=$(ls -t "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products | grep pipeline- | head -1)
if [ -n "$WEBLOG_DIR" ]
then
    # if weblog.tgz exists, we want to remove it
    if [ -e "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products/weblog.tgz ]
    then
        rm -f "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products/weblog.tgz
    fi

    # Tar & Zip the weblog
    tar -C "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products -czf "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products/weblog.tgz "${WEBLOG_DIR}"

    if [ $? -ne 0 ]
    then
        echo "Creation of weblog.tgz failed, exiting"
        exit 1
    fi
else
    # no weblog directory.  If there's no weblog.tgz file, there's an issue: Issue a warning
    if [ ! -e "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products/weblog.tgz ]
    then
        echo "Neither weblog.tgz or the weblog directory exist, continuing"
    fi
fi

#
# Sanity checks: create a staging subdirectory for this cal, and if the file already exists, remove it.
#
mkdir -p "${STAGING_DIR}"/"${WORKFLOW_DIR}"
if [ -e "${STAGING_DIR}"/"${WORKFLOW_DIR}"/"${FILENAME}".tar ]
then
    echo "Calibration Tar File Already Exists!  Removing the file for recreation"
    /bin/rm -f "${STAGING_DIR}"/"${WORKFLOW_DIR}"/"${FILENAME}".tar
fi

#
# tar all non-fits and non-weblog-related files into a tar archive in the storage path
# SSA-6115:  Don't exclude the weblog.tgz: Users and DAs prefer it bundled in.
#
tar --exclude=\*.fits --exclude=pipeline-\* -C "${SPOOL_DIR}"/"${WORKFLOW_DIR}" -cvf "${STAGING_DIR}"/"${WORKFLOW_DIR}"/"${FILENAME}".tar products
if [ $? -ne 0 ]
then
    echo "Creation of main tar file failed, exiting"
    exit 1
fi

#
# Copy the weblog over, for ingestion as an ancillary file
#
/bin/cp -f "${SPOOL_DIR}"/"${WORKFLOW_DIR}"/products/weblog.tgz "${STAGING_DIR}"/"${WORKFLOW_DIR}"
if [ $? -ne 0 ]
then
    echo "Copy of the weblog to staging location failed.  Exiting."
    exit 1
fi

#
# To stay consistent with current working methods: Copy from STAGING_DIR to STORAGE_DIR
#
mkdir -p "${STORAGE_DIR}"/"${WORKFLOW_DIR}"
cp "${STAGING_DIR}"/"${WORKFLOW_DIR}"/"${FILENAME}".tar "${STORAGE_DIR}"/"${WORKFLOW_DIR}"/"${FILENAME}".tar
if [ $? -ne 0 ]
then
    # If something goes wrong, make some noise, but continue on.
    echo "Failed to copy the calibration to ${STORAGE_DIR}, continuing."
    $? = 0
fi
