#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Test data for two VLA EBs, a calibration product, and VLBA and VLASS
    image products
"""
import datetime
import warnings

from sqlalchemy import exc as sa_exc

from schema import Author, ExecutionBlock, Project, ScienceProduct

from ..utilities import (
    DATE_FORMAT,
    DATETIME_FORMAT,
    get_exec_block_details_from_loc_report,
    get_file_info_from_loc_report,
    get_locations_report,
)

# pylint: disable=C0301, R0201

METADATA_INGESTION_DATE = datetime.datetime.strptime("2019-12-12 16:04:02.11191", DATETIME_FORMAT)
EXECUTION_BLOCK = "Execution Block"
PPR_FILENAME = "PPR.xml"
CASA_LOG = "casa_commands.log"
MANIFEST = "unknown.pipeline_manifest.xml"
DO_NOT_CALIBRATE = "Do Not Calibrate"
VLBA_OPERATIONS = "VLBA Operations"


class VlaProductSet:
    """Encapsulates project 17A-109's products"""

    def __init__(self):
        self.project = self.build_project()
        self.exec_blocks = self.build_exec_blocks()
        self.science_products = self.build_science_products()
        self.file_info, self.total_size = get_exec_block_details_from_loc_report("17a-109_fg_", self.exec_blocks)
        self.measurement_set = self.build_basic_ms()
        self.cms = self.build_cms()

    def build_project(self) -> Project:
        """Construct VlaProductSet for 17A-109"""

        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)

            project = Project(
                project_code="17A-109",
                starttime=57795.8403622685,
                endtime=57892.7843396991,
                title="A detailed study of the jet-galaxy interaction in Minkowski's Object",
                abstract="Minkowski's Object is a dwarf starforming galaxy interacting with an FRI radio jet from a nearby elliptical galaxy. It has been proposed as a prototype for jet-induced star formation, a hypothesis consistent with our recent ALMA observations that map the CO 1-0 emission in the galaxy. In order to better understand the nature of the interaction, we are requesting C- and Ku-band observations at matched resolution to our ALMA and archival VLA L-band data. Faraday synthesis of the C-band polarimetric data will allow us to study the geometry of the interaction, in particular how well the relativistic electron population is mixed with the ionized gas in the star forming region. This will give us important information for our simulations of the interaction and the triggering of star formation in Minkowski's Object. In turn, this will inform on the conditions where jet-induced star formation is likely in higher redshift objects. We will use the combination of the C- and Ku-band data to investigate the variation the spectral index in the interaction region, in particular to search for shock acceleration in the interaction zone.",
                proprietary_duration=365,
                total_observation_time=0.0391880786919501,
                legacy_id="AL988",
                opt_project_id=33151323,
                last_addition=datetime.datetime.strptime("2017-08-28", DATE_FORMAT).date(),
            )
            project.authors = self.build_authors(project.project_code)
        return project

    def build_authors(self, project_code: str) -> list:
        """Returns author details for this project."""

        authors = [
            Author(
                project_code=project_code,
                author_id=780,
                username="mlacy",
                firstname="Mark",
                lastname="Lacy",
                pst_person_id="885",
                is_pi=True,
            ),
            Author(
                project_code=project_code,
                author_id=781,
                username="swood",
                firstname="Sarah",
                lastname="Wood",
                pst_person_id="6877",
                is_pi=False,
            ),
            Author(
                project_code=project_code,
                author_id=782,
                username="stevecroft",
                firstname="Steve",
                lastname="Croft",
                pst_person_id="2116",
                is_pi=False,
            ),
            Author(
                project_code=project_code,
                author_id=783,
                username="KristinaNyland",
                firstname="Kristina",
                lastname="Nyland",
                pst_person_id="2950",
                is_pi=False,
            ),
            Author(
                project_code=project_code,
                author_id=784,
                username="fragilep",
                firstname="Chris",
                lastname="Fragile",
                pst_person_id="9238",
                is_pi=False,
            ),
        ]
        return authors

    def build_exec_blocks(self) -> list:
        """Returns this project's execution blocks"""

        exec_blocks = [
            ExecutionBlock(
                execution_block_id=910,
                filegroup_id=18468,
                ngas_fileset_id="17A-109.sb33151327.eb33496982.57795.84034732639",
                scheduling_block_id=33151327,
                scheduling_block_type="OBSERVER",
                ost_exec_block_id=33496982,
                configuration="D",
                starttime=57795.8403622685,
                endtime=57795.8795503472,
                calibration_status="Calibrated",
                band_code="KU X",
            ),
            ExecutionBlock(
                execution_block_id=910,
                filegroup_id=41979,
                ngas_fileset_id="17A-109.sb33151331.eb33786546.57892.65940042824",
                scheduling_block_id=33151331,
                scheduling_block_type="OBSERVER",
                ost_exec_block_id=33786546,
                configuration="C",
                starttime=57892.6600277778,
                endtime=57892.7843396991,
                calibration_status="Calibrated",
                band_code="C X",
            ),
        ]
        return exec_blocks

    def build_science_products(self) -> list:
        """Returns this project's science products"""

        science_products = [
            ScienceProduct(
                filegroup_id=18468,
                external_name="17A-109.sb33151327.eb33496982.57795.84034732639",
                metadata_ingestion_date=METADATA_INGESTION_DATE,
                metadata_ingestion_version="1",
                science_product_type=EXECUTION_BLOCK,
            ),
            ScienceProduct(
                filegroup_id=41979,
                external_name="17A-109.sb33151331.eb33786546.57892.65940042824",
                metadata_ingestion_date=METADATA_INGESTION_DATE,
                metadata_ingestion_version="1",
                science_product_type=EXECUTION_BLOCK,
            ),
        ]
        return science_products

    def build_basic_ms(self) -> dict:
        """
        Returns measurement set metadata for a couple of this project's
        filegroups

        :return:
        """

        files = {
            "fg_41979": [
                {"filename": "weblog.tgz", "size": 1683135},
                {"filename": "unknown.pprequest.xml", "size": 5611},
                {"filename": "pipeline_aquareport.xml", "size": 1351},
                {"filename": PPR_FILENAME, "size": 5611},
                {"filename": "unknown.auxproducts.tgz", "size": 615},
                {"filename": CASA_LOG, "size": 1995},
                {"filename": MANIFEST, "size": 809},
                {"filename": "casa_pipescript.py", "size": 823},
                # the tar delivered when basic MS is requested
                {
                    "filename": "17A-109.sb33151327.eb33496982.57795.84034732639.ms.tgz",
                    "size": 23690977245,
                },
            ],
            "fg_18468": [
                {"filename": "weblog.tgz", "size": 1681803},
                {"filename": "unknown.pprequest.xml", "size": 5610},
                {"filename": "pipeline_aquareport.xml", "size": 1351},
                {"filename": PPR_FILENAME, "size": 5610},
                {"filename": "unknown.auxproducts.tgz", "size": 615},
                {"filename": CASA_LOG, "size": 1995},
                {"filename": MANIFEST, "size": 809},
                {"filename": "casa_pipescript.py", "size": 823},
                # the tar delivered when basic MS is requested
                {
                    "filename": "17A-109.sb33151327.eb33496982.57795.84034732639.ms.tgz",
                    "size": 23690977531,
                },
            ],
        }
        return files

    def build_cms(self):
        """
        Returns calibrated measurement set metadata
        for a couple of this project's filegroups

        :return:
        """

        files = {
            "fg_41979": [
                {
                    "filename": "17A-109.sb33151331.eb33786546.57892.65940042824.ms" ".calapply.txt",
                    "size": 1058,
                },
                {"filename": MANIFEST, "size": 11006},
                {"filename": "unknown.session_1.caltables.tgz", "size": 71653397},
                {"filename": "casa_piperestorescript.py", "size": 205},
                {"filename": "flux.csv", "size": 22},
                {"filename": PPR_FILENAME, "size": 1632},
                {"filename": CASA_LOG, "size": 133935},
                # the tar delivered when calibrated MS (aka 'restore') is
                # requested
                {
                    "filename": "17A-109.sb33151331.eb33786546.57892.65940042824.400685418.tar.gz",
                    "size": 88842463132,
                },
            ],
            "fg_18468": [
                {"filename": MANIFEST, "size": 24025},
                {"filename": "unknown.session_1.caltables.tgz", "size": 26463229},
                {"filename": "casa_piperestorescript.py", "size": 1149},
                {"filename": "flux.csv", "size": 22},
                {"filename": PPR_FILENAME, "size": 1632},
                {"filename": CASA_LOG, "size": 260953},
                {
                    "filename": "17A-109.sb33151327.eb33496982.57795.84034732639.ms.calapply.txt",
                    "size": 1058,
                },
                {
                    "filename": "17A-109.sb33151327.eb33496982.57795.84034732639.400820221.tar.gz",
                    "size": 47520134173,
                },
            ],
        }

        return files


class CalibrationProduct:
    """Encapsulates calibration test data for 18B-265"""

    def __init__(self):
        self.project = self.build_project()
        self.science_product = self.build_science_product()
        self.file_info, self.total_size = get_file_info_from_loc_report(get_locations_report("calibration"))

    def build_project(self) -> Project:
        """
        Returns 18B-265 Project.

        :return:

        """

        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)

            project = Project(
                project_code="18B-265",
                legacy_id="AS1535",
                total_observation_time=0.0316695485962555,
                proprietary_duration=365,
                starttime=58745.9972239583,
                endtime=59043.2400404977,
                last_addition=datetime.datetime.strptime("2020-07-13", DATE_FORMAT).date(),
            )
            project.authors = self.build_authors()
        return project

    def build_authors(self) -> list:
        """
        Returns author details for 18B-265
        :return:

        """

        return [
            Author(
                project_code="18B-265",
                author_id=59386,
                username="mansi",
                firstname="Mansi",
                lastname="Kasliwal",
                pst_person_id="8998",
                is_pi=False,
            ),
            Author(
                project_code="18B-265",
                author_id=59387,
                username="chomiuk",
                firstname="Laura",
                lastname="Chomiuk",
                pst_person_id="701",
                is_pi=False,
            ),
            Author(
                project_code="18B-265",
                author_id=59388,
                username="kirx",
                firstname="Kirill",
                lastname="Sokoloski",
                pst_person_id="5512",
                is_pi=False,
            ),
            Author(
                project_code="18B-265",
                author_id=59389,
                username="KojiMukai",
                firstname="Koji",
                lastname="Muki",
                pst_person_id="3674",
                is_pi=False,
            ),
            Author(
                project_code="18B-265",
                author_id=59387,
                username="JustinLinford",
                firstname="Justin",
                lastname="Linford",
                pst_person_id="3794",
                is_pi=False,
            ),
            Author(
                project_code="18B-265",
                author_id=59390,
                username="ThomasNelson",
                firstname="Thomas",
                lastname="Nelson",
                pst_person_id="5234",
                is_pi=False,
            ),
            Author(
                project_code="18B-265",
                author_id=59394,
                username="EliasAydi",
                firstname="Elias",
                lastname="Aydi",
                pst_person_id="10794",
                is_pi=False,
            ),
            Author(
                project_code="18B-265",
                author_id=59392,
                username="jeno@astro.columbia.edu",
                firstname="Jennifer",
                lastname="Sokoloski",
                pst_person_id="1200",
                is_pi=True,
            ),
            Author(
                project_code="18B-265",
                author_id=59393,
                username="amkawash",
                firstname="Adam",
                lastname="Kawash",
                pst_person_id="8687",
                is_pi=False,
            ),
        ]

    def build_science_product(self):
        """
        Returns the science product associated with this calibration.

        :return:
        """

        return ScienceProduct(
            filegroup_id=388273,
            external_name="18B-265_2019_12_10_T00_00_59.203.tar",
            science_product_type="Calibration",
            metadata_ingestion_version="1",
            metadata_ingestion_date=datetime.datetime.strptime("2019-12-13 23:30:57.730581", DATETIME_FORMAT),
            external_system="EVLA Processing",
        )


class VlbaProductSet:

    """Encapsulates a set of products for VLBA project BT142."""

    def __init__(self):
        self.project = self.build_project()
        self.exec_blocks = self.build_exec_blocks()
        self.science_products = self.build_science_products()
        self.file_info, self.total_size = self.get_vlba_file_info_from_loc_report()

    def build_project(self) -> Project:
        """
        Returns BT142 metadata and authors details

        :return:
        """

        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)

            project = Project(
                project_code="BT142",
                starttime=58154.8048148148,
                endtime=58217.8342361111,
                title="Jets, outflows, and inclined disks in Seyfert galaxies: the H2O megamasers' view",
                abstract="Jets, outflows, and inclined disks in Seyfert galaxies: the H2O megamasers' view | Candidate extragalactic water masers associated with accretion disks around AGN are typically identified by a triple-peak profile of the maser spectra. However, a large number of maser sources show single-broad lines or groups of lines clustered in a narrow velocity range. In this proposal, we propose for a study of four galaxies, classified as Seyfert or LINER, hosting bright water masers and showing evidences of ejection/accretion nuclear activity. Our main goals are that of detecting the maser emission at VLBI scales, determining its location and distribution with respect to the main nuclear centers of activity, and derive the nature of the emission. These sources will potentially constitute new case studies for the scarcely-populated class of confirmed jet/outflow masers or very first examples of the exotic, recently-invoked class of 'inclined water maser disks'. Furthermore, the proposed mesurements will set the basis for a number of follow-up (VLBI) studies aim at a better understanding of the physics and disk/jet geometry in Seyferts and LINERs.",
                proprietary_duration=365,
                total_observation_time=0.0391880786919501,
                last_addition=datetime.datetime.strptime("2018-11-05", DATE_FORMAT).date(),
            )
            project.authors = self.build_authors()

        return project

    def build_authors(self) -> list:
        """
        Returns author details for this project
        :return:
        """

        authors = [
            Author(
                username="atarchi",
                firstname="Andrea",
                lastname="Tarchi",
                pst_person_id="236",
                project_code="BT142",
                author_id=45311,
                is_pi=True,
            ),
            Author(
                username="jbraatz",
                firstname="James",
                lastname="Braatz",
                pst_person_id="26",
                project_code="BT142",
                author_id=45310,
                is_pi=False,
            ),
            Author(
                username="pcastangia",
                firstname="Paola",
                lastname="Castangia",
                pst_person_id="541",
                project_code="BT142",
                author_id=45312,
                is_pi=False,
            ),
            Author(
                username="gsurcis",
                firstname="Gabriele",
                lastname="Surcis",
                pst_person_id="2512",
                project_code="BT142",
                author_id=45313,
                is_pi=False,
            ),
        ]
        return authors

    def build_exec_blocks(self) -> list:
        """
        Constructs this project's execution blocks.

        :return:
        """

        exec_blocks = [
            ExecutionBlock(
                execution_block_id=108778,
                filegroup_id=278938,
                starttime=58175.0757060185,
                endtime=58175.325625,
                calibration_status=DO_NOT_CALIBRATE,
                band_code="K",
            ),
            ExecutionBlock(
                execution_block_id=108833,
                filegroup_id=279054,
                starttime=58160.1166550926,
                endtime=58160.3043634259,
                calibration_status=DO_NOT_CALIBRATE,
                band_code="L",
            ),
            ExecutionBlock(
                execution_block_id=108862,
                filegroup_id=279119,
                starttime=58154.8048148148,
                endtime=58154.9928009259,
                calibration_status=DO_NOT_CALIBRATE,
                band_code="L",
            ),
            ExecutionBlock(
                execution_block_id=108630,
                filegroup_id=278613,
                starttime=58217.5843171296,
                endtime=58217.8342361111,
                calibration_status=DO_NOT_CALIBRATE,
                band_code="K",
            ),
        ]
        return exec_blocks

    def build_science_products(self):
        """
        Returns science products associate with the exec blocks we got

        :return:
        """

        return [
            ScienceProduct(
                filegroup_id=279054,
                science_product_type=EXECUTION_BLOCK,
                metadata_ingestion_version="1",
                metadata_ingestion_date=METADATA_INGESTION_DATE,
                external_name="BT142A2",
                external_system=VLBA_OPERATIONS,
            ),
            ScienceProduct(
                filegroup_id=278938,
                science_product_type=EXECUTION_BLOCK,
                metadata_ingestion_version="1",
                metadata_ingestion_date=METADATA_INGESTION_DATE,
                external_name="BT142A1",
                external_system=VLBA_OPERATIONS,
            ),
            ScienceProduct(
                filegroup_id=279119,
                science_product_type=EXECUTION_BLOCK,
                metadata_ingestion_version="1",
                metadata_ingestion_date=METADATA_INGESTION_DATE,
                external_name="BT142B2",
                external_system=VLBA_OPERATIONS,
            ),
            ScienceProduct(
                filegroup_id=278613,
                science_product_type=EXECUTION_BLOCK,
                metadata_ingestion_version="1",
                metadata_ingestion_date=METADATA_INGESTION_DATE,
                external_name="BT142B1",
                external_system=VLBA_OPERATIONS,
            ),
        ]

    def get_vlba_file_info_from_loc_report(self) -> tuple:
        """
        Pull file metadata from VLBA locations report.
        We're using this metadata differently from EVLA metadata;
        hence the bespoke method.

        :return:
        """

        file_info = dict()
        locations_report = get_locations_report("vlba_eb")

        for file_spec in locations_report["files"]:
            filename = file_spec["ngas_file_id"]
            size = file_spec["size"]
            file_info[filename] = size

        return file_info, locations_report["aggregate_size"]


class VlassImageProductSet:
    """Encapsulates a VLASS image product"""

    def __init__(self):
        self.project = self.build_project()
        self.science_products = self.build_science_products()
        location_report = get_locations_report("img")
        self.file_info, self.total_size = get_file_info_from_loc_report(location_report)

    def build_project(self) -> Project:
        """
        Returns VLASS1.1 project metadata

        :return:
        """

        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)

            project = Project(
                project_code="VLASS1.1",
                title="The Very Large Array Sky Survey",
                abstract='The Very Large Array Sky Survey (VLASS) is a 5500-hr, community-driven project to survey the whole sky visible to the VLA. It will engage radio astronomy experts, multi-wavelength astronomers and citizen scientists alike. The data will be taken in three passes over the sky to allow the discovery of transient radio sources, and will cover the frequency range 2-4 GHz with an angular resolution of 2.5 arcsec. By utilizing the "on the fly" interferometry mode, the overheads will be much reduced compared to conventional survey techniques. The key science topics to be addressed by the survey are: Imaging Galaxies Through Time and Space; Hidden Explosions; Faraday Tomography of The Magnetic Sky; Peering Though Our Dusty Galaxy; and Missing Physics.',
                opt_project_id=33997662,
                starttime=58004.0162552083,
                endtime=58169.6566603472,
                total_observation_time=0.158181712955411,
                last_addition=datetime.datetime.strptime("2017-09-08", DATE_FORMAT).date(),
                proprietary_duration=0,
            )
            project.authors = self.build_authors()
        return project

    def build_authors(self) -> list:
        """Returns VLASS1.1 authors' metadata"""

        return [
            Author(
                project_code="VLASS1.1",
                username="vlass",
                firstname="Vlass",
                lastname="Scientist",
                pst_person_id="9707",
                is_pi=True,
            ),
            Author(
                project_code="VLASS1.1",
                username="cchandle",
                firstname="Claire",
                lastname="Chandler",
                pst_person_id="36",
                is_pi=False,
            ),
            Author(
                project_code="VLASS1.1",
                username="akimball",
                firstname="Amy",
                lastname="Kimball",
                pst_person_id="1743",
                is_pi=False,
            ),
            Author(
                project_code="VLASS1.1",
                username="jwrobel",
                firstname="Joan",
                lastname="Wrobel",
                pst_person_id="172",
                is_pi=False,
            ),
            Author(
                project_code="VLASS1.1",
                username="fschinzel",
                firstname="Frank",
                lastname="Schinzel",
                pst_person_id="2059",
                is_pi=False,
            ),
        ]

    def build_science_products(self) -> list:
        """
        Returns science product associated with this image file,
        as a list for consistency with the other ProductSets.

        :return:
        """

        return [
            ScienceProduct(
                filegroup_id=223148,
                external_name="VLASS1.1.ql.T01t01.J000232-383000.10.2048.v1",
                metadata_ingestion_version="1",
                metadata_ingestion_date=METADATA_INGESTION_DATE,
                science_product_type="Image",
                external_system="CASA Pipeline",
            ),
        ]
